package org.apache.hadoop.mapred.adaptive;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.apache.hadoop.mapred.JobTracker;

public class Reputation {

	public Map<String, ArrayList<Gossip>> gossipQueues = new HashMap<String, ArrayList<Gossip>>();
	public Map<String, Integer> taskTrackerToReputation = new HashMap<String, Integer>();
	public Map<String, HashSet<String>> diversity = new HashMap<String, HashSet<String>>();
	public int threshold;

	public int REPUTATION_INITIAL = 0;
	public int REPUTATION_MAX = 30;
	public int REPUTATION_MIN = -10;
	public int PENALTY_INITIAL = 2;
	public int INCREAMENTAL_RATE = 2;

	public int getReputation(String host) {
		return taskTrackerToReputation.get(host);
	}

	public void calReputationIncrease(String host) {
		Integer rp = this.taskTrackerToReputation.get(host);
		// 增长
		rp = (rp == null) ? 1 : (rp + 1);
		if (rp > REPUTATION_MAX)
			rp = rp - 1;
		taskTrackerToReputation.put(host, rp);
	}

	public boolean calReputation(String from, String to) {

		// prepare
		HashSet<String> div = this.diversity.get(to);
		if (null == div) {
			div = new HashSet<String>();
			this.diversity.put(to, div);
		}
		ArrayList<Gossip> gossipQueue = this.gossipQueues.get(to);
		if (null == gossipQueue) {
			gossipQueue = new ArrayList<Gossip>();
			this.gossipQueues.put(to, gossipQueue);
		}

		// process
		Integer rp = this.taskTrackerToReputation.get(to);
		int penalty = 0;
		Gossip gossip;
		if (!div.contains(from)) {
			div.add(from);
			penalty = div.size() * PENALTY_INITIAL;
		} else {
			for (Gossip go : gossipQueue) {
				if (go.from.equals(from)) {
					penalty = go.penalty * INCREAMENTAL_RATE;
					break;
				}
			}
		}
		gossip = new Gossip(from, to, penalty);
		gossipQueue.add(0, gossip);
		rp = rp - penalty;
		taskTrackerToReputation.put(to, rp);

		if (rp < REPUTATION_MIN)
			return true;
		else return false;
	}

	public int getGossipQueueSize(String host) {
		ArrayList<Gossip> gossipQueue = this.gossipQueues.get(host);
		if (null != gossipQueue)
			return this.gossipQueues.get(host).size();
		else
			return 0;
	}
	
	public String getGossips(String host) {
		ArrayList<Gossip> gossipQueue = this.gossipQueues.get(host);
		if (null != gossipQueue && gossipQueue.size() != 0){
			String result = "";
			for(int i = 0 ; i < gossipQueue.size() ; i ++ ){
				result += i + ". " + gossipQueue.get(i).toString();
			}
			return result;
		}
		else
			return "";
	}
	
/*	public static void main(String args[]){
		Reputation rp = new Reputation();
		
		String name[] = {"1","2","3","4","5","6"};
		
		for(String n : name){
			for(int i = 0; i< 30 ; i++)rp.calReputationIncrease(n);
		}
		
		rp.calReputation(name[0], name[2]);
		
		rp.calReputation(name[1], name[2]);
		rp.calReputation(name[0], name[2]);
		rp.calReputation(name[1], name[2]);
		
		rp.calReputation(name[3], name[2]);
		rp.calReputation(name[4], name[2]);
		
		rp.calReputation(name[5], name[2]);
		rp.calReputation(name[0], name[2]);
		rp.calReputation(name[0], name[2]);
		
		System.out.println(rp.taskTrackerToReputation.get(name[2]));
		
	}*/
}
